@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -55,7 +55,7 @@ rhuber <- function(n, k = 0.862, mu = 0, sigma = 1) {
                            length.arg = 1, positive = TRUE))
               stop("bad input for argument 'n'") else n
 
-  myl <- rep(0.0, len = use.n)
+  myl <- rep_len(0.0, use.n)
 
   lowlim <- 1
   upplim <- 0
@@ -72,15 +72,15 @@ rhuber <- function(n, k = 0.862, mu = 0, sigma = 1) {
       upplim <- upplim + sumyok
 
       if (upplim > use.n)
-        myl <- rep(myl, len = upplim)
+        myl <- rep_len(myl, upplim)
 
       myl[lowlim:upplim] <- y[yok]
       lowlim <- lowlim + sumyok
     }
   }
-  myl <- rep(myl, len = use.n)  # Prune to right length
+  myl <- rep_len(myl, use.n)  # Prune to right length
 
-  rep(mu + sigma * myl, len = use.n)
+  rep_len(mu + sigma * myl, use.n)
 }
 
 
@@ -97,14 +97,14 @@ qhuber <- function (p, k = 0.862, mu = 0, sigma = 1,
                     lower.tail = TRUE, log.p = FALSE ) {
 
   if (!is.logical(lower.tail) || length(lower.tail ) != 1)
-    stop("bad input for argument 'lower.tail'") 
+    stop("bad input for argument 'lower.tail'")
   if (!is.logical(log.p) || length(log.p) != 1)
     stop("bad input for argument 'log.p'")
 
   cnorm <- sqrt(2 * pi) * ((2 * pnorm(k) - 1) + 2 * dnorm(k) / k)
   if (lower.tail) {
     if (log.p) {
-      ln.p <- p 
+      ln.p <- p
       x <- pmin(exp(ln.p), -expm1(ln.p))
     } else {
       x <- pmin(p, 1 - p)
@@ -113,7 +113,7 @@ qhuber <- function (p, k = 0.862, mu = 0, sigma = 1,
     if (log.p) {
       ln.p <- p
       x <- pmin(-expm1(ln.p), exp(ln.p))
-    } else { 
+    } else {
       x <- pmin(1 - p, p)
     }
   }
@@ -129,7 +129,7 @@ qhuber <- function (p, k = 0.862, mu = 0, sigma = 1,
     } } else {
       if (log.p) {
         ifelse(exp(ln.p) > 0.5, mu + q * sigma, mu - q * sigma)
-      } else { 
+      } else {
         ifelse(p > 0.5, mu + q * sigma, mu - q * sigma)
       }
     }
@@ -144,7 +144,7 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
                    lower.tail = TRUE, log.p = FALSE ) {
 
   if (!is.logical(lower.tail) || length(lower.tail ) != 1)
-    stop("bad input for argument 'lower.tail'") 
+    stop("bad input for argument 'lower.tail'")
   if (!is.logical(log.p) || length(log.p) != 1)
     stop("bad input for argument 'log.p'")
 
@@ -155,7 +155,7 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
   p <- ifelse(x <= -k ,
               exp(k^2 / 2) / k * exp(k * x) / sqrt(2 * pi),
               dnorm(k) / k + pnorm(x) - pnorm(-k))
-  
+
   if (lower.tail) {
     if (log.p) {
       ans <- ifelse(zedd <= 0, log(p) + log1p(-eps),
@@ -172,7 +172,7 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
       ans <- ifelse(zedd <= 0, -expm1(log(p) + log1p(-eps)),
                                exp(log(p) + log1p(-eps)))
     }
-  } 
+  }
   ans
 }
 
@@ -181,7 +181,8 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
 
 
  huber2 <- function(llocation = "identitylink", lscale = "loge",
-                    k = 0.862, imethod = 1, zero = 2) {
+                    k = 0.862, imethod = 1,
+                    zero = "scale") {
 
 
   A1 <- (2 * dnorm(k) / k - 2 * pnorm(-k))
@@ -195,9 +196,6 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
   if (!is.Numeric(k, length.arg = 1, positive = TRUE))
     stop("bad input for argument 'k'")
 
-  if (length(zero) &&
-      !is.Numeric(zero, integer.valued = TRUE, positive = TRUE))
-    stop("bad input for argument 'zero'")
 
 
   llocat <- as.list(substitute(llocation))
@@ -216,9 +214,27 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
             namesof("location",  llocat,  earg = elocat), ", ",
             namesof("scale",     lscale,  earg = escale), "\n\n",
             "Mean: location"),
+
   constraints = eval(substitute(expression({
-    constraints <- cm.zero.VGAM(constraints, x = x, .zero , M = M)
+    constraints <- cm.zero.VGAM(constraints, x = x, .zero , M = M,
+                                predictors.names = predictors.names,
+                                M1 = 2)
   }), list( .zero = zero ))),
+
+  infos = eval(substitute(function(...) {
+    list(M1 = 2,
+         Q1 = 1,
+         expected = TRUE,
+         multipleResponses = FALSE,
+         parameters.names = c("location", "scale"),
+         llocation = .llocat ,
+         lscale    = .lscale ,
+         zero = .zero )
+  }, list( .zero = zero,
+           .llocat = llocat,
+           .lscale = lscale ))),
+
+
   initialize = eval(substitute(expression({
 
     temp5 <-
@@ -239,9 +255,9 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
       scale.y.est <- sqrt( sum(c(w) * junk$resid^2) / junk$df.residual )
       location.init <- if ( .llocat == "loge") pmax(1/1024, y) else {
         if ( .imethod == 3) {
-          rep(weighted.mean(y, w), len = n)
+          rep_len(weighted.mean(y, w), n)
         } else if ( .imethod == 2) {
-          rep(median(rep(y, w)), len = n)
+          rep_len(median(rep(y, w)), n)
         } else if ( .imethod == 1) {
           junk$fitted
         } else {
@@ -292,6 +308,15 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
            .elocat = elocat, .escale = escale,
            .k      = k ))),
   vfamily = c("huber2"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    mylocat <- eta2theta(eta[, 1], .llocat ,  earg = .elocat )
+    myscale <- eta2theta(eta[, 2], .lscale ,  earg = .escale )
+    okay1 <- all(is.finite(mylocat)) &&
+             all(is.finite(myscale)) && all(0 < myscale)
+    okay1
+  }, list( .llocat = llocat, .lscale = lscale,
+           .elocat = elocat, .escale = escale,
+           .k      = k ))),
   deriv = eval(substitute(expression({
     mylocat <- eta2theta(eta[, 1], .llocat ,  earg = .elocat )
     myscale <- eta2theta(eta[, 2], .lscale ,  earg = .escale )
@@ -321,7 +346,7 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
             .elocat = elocat, .escale = escale,
             .eps    = eps,       .k      = k ))),
   weight = eval(substitute(expression({
-    wz   <- matrix(as.numeric(NA), n, 2)  # diag matrix; y is one-col too
+    wz   <- matrix(NA_real_, n, 2)  # diag matrix; y is one-col too
 
 
 
@@ -388,9 +413,9 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
       junk <- lm.wfit(x = x, y = c(y), w = c(w))
       location.init <- if ( .llocat == "loge") pmax(1/1024, y) else {
         if ( .imethod == 3) {
-          rep(weighted.mean(y, w), len = n)
+          rep_len(weighted.mean(y, w), n)
         } else if ( .imethod == 2) {
-          rep(median(rep(y, w)), len = n)
+          rep_len(median(rep(y, w)), n)
         } else if ( .imethod == 1) {
           junk$fitted
         } else {
@@ -440,6 +465,13 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
            .elocat = elocat,
            .k      = k ))),
   vfamily = c("huber1"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    mylocat <- eta2theta(eta, .llocat ,  earg = .elocat )
+    okay1 <- all(is.finite(mylocat))
+    okay1
+  }, list( .llocat = llocat,
+           .elocat = elocat,
+           .k      = k ))),
   deriv = eval(substitute(expression({
     mylocat <- eta2theta(eta, .llocat ,  earg = .elocat )
     myk     <- .k
@@ -468,7 +500,7 @@ phuber <- function(q, k = 0.862, mu = 0, sigma = 1,
             .elocat = elocat,
             .eps    = eps,       .k      = k ))),
   weight = eval(substitute(expression({
-    wz   <- matrix(as.numeric(NA), n, 1)  # diag matrix; y is one-col too
+    wz   <- matrix(NA_real_, n, 1)  # diag matrix; y is one-col too
 
 
 
